/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023

 Date          : February 2009

 File          : SPI_Master.c

 Hardware      : Applicable to ADuC7023 rev A later silicon
                 
 Description   : This SPI Master example shows how to initialise the SPI peripheral in Master mode
 			This simple example transmits 6 bytes to a Slave after a reset.
			When an XIRQ0 interrupt occurs, the Master will then read back 6 bytes from the Slave.
				

*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT22 0x400000
#define BIT23 0x800000
#define BIT25 0x2000000
#define BIT30 0x40000000
#define BIT31 0x80000000

#include <aduc7023.h>


unsigned char szTxData[] = {0x1, 0x2, 0x10, 0x20, 0x40, 0x80};	// Array to send to Slave
unsigned char ucTxCount = 0;  					// Array index variable for szTxData[]
unsigned char szRxData[16];	  					// Array for reading Data from Slave
unsigned char ucRxCount = 0;  					// Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ0 = 1; 			// Flag to begin reading from Slave - controlled by XIRQ0
unsigned char ucDummyRd = 0;
unsigned char ucTest = 0;
int main(void) 
{
	POWKEY1 = 0x01;								// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	

	POWKEY3 = 0x76;			 					// Enable PWM, SPI, I2C0, I2C1 Peripherals with 41.78MHz clock
    POWCON1 = 0x924;					
	POWKEY4 = 0xB1;

	// Configure P0.6 as MISO, P0.7 as MOSI, 
	// Configure P1.0 as SCLOCK and P1.1 as /SS
	GP0CON |= 0x11000000;						// Only write to bits 7,6
	GP1CON |= 0x11;								// Only write to bits 1,0
	SPICON = BIT0 + BIT1   						// Enable SPI + Enable SPI Master
		    + BIT6  							// Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
			+ BIT9								// Enable MISO output driver.
		   + BIT11;		   						//Continuous transfer mode + Enable IRQ on 2 byte transfer
	SPIDIV  = 0xFC;	   							// Select 101kHz clock rate = 0x31
	IRQCONE = BIT1;								// Rising edge of XIRQ0
	IRQEN = BIT12 + BIT13; 						// Enable SPI + XIRQ0 interrupt
	// Begin Master Transmit sequence
	ucTxCount = 0;
	SPITX = szTxData[ucTxCount++];
	
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
	while (1)
	{	 	
		if (ucWaitingForXIRQ0 == 0)				// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ0 = 1;
			ucRxCount = 0;
			SPICON |= BIT12; 					// Flush Rx FIFO
			SPICON &= ~BIT12; 
			SPICON &= ~BIT6; 					// Generate Interrupt on receiving data
		//	SPICON |= BIT14 + BIT15;
			SPITX = 0x71;
			ucDummyRd = SPIRX;
		}
	}
			
}
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int SPIMSTATUS = 0;


	IRQSTATUS = IRQSTA;	   						// Read off IRQSTA register
	if ((IRQSTATUS & BIT13) == BIT13)			//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;					// Enable Read request from Slave.
		IRQCLRE = BIT13;
		IRQCLR = BIT13;
	}

	if ((IRQSTATUS & BIT12) == BIT12)			//If SPI Master interrupt source
	{
	   SPIMSTATUS = SPISTA;
	   if ((SPIMSTATUS & BIT4) == BIT4) 		// SPI Tx underflow
	   {
			
	   }
	   if ((SPIMSTATUS & BIT5) == BIT5) 		// If SPI Master Tx IRQ
	   {
			if (ucTxCount < 7)		    		// Have max 6 bytes been sent?
				SPITX = szTxData[ucTxCount++];	// Load Tx buffer
	   }
	   if ((SPIMSTATUS & BIT6) == BIT6) 		// If SPI Master Rx IRQ
	   {
			ucTest++;
			GP1DAT ^= BIT22;					// Toggle LED on Evaluation board (P1.6)
			if (ucRxCount < 16)		    	  	// Have max 6 bytes been received?
			{
				szRxData[ucRxCount] = SPIRX;  	// Read Rx buffer
				ucRxCount++;

			}
			else
				ucRxCount = 0;
	   }
	}
}

